"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = exports.Shard = void 0;
const tslib_1 = require("tslib");
const buffer_1 = require("buffer");
const cid_1 = require("multiformats/cid");
class Shard {
    constructor(ipfs, cid, context) {
        this.ipfs = ipfs;
        this.cid = cid;
        this.context = context;
    }
    static encode(data) {
        return buffer_1.Buffer.from(JSON.stringify(data), 'utf-8');
    }
    static decode(data) {
        return JSON.parse(data.toString('utf-8'));
    }
    static from(ipfs, cid, context) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return new Shard(ipfs, cid, context);
        });
    }
    static create(ipfs, data, context) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const cid = yield ipfs.object.put({ Data: Shard.encode(data), Links: [] });
            return new Shard(ipfs, cid, context);
        });
    }
    get() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const pbNode = yield this.ipfs.object.get(this.cid);
            if (!pbNode.Data)
                return null;
            return Shard.decode(pbNode.Data);
        });
    }
    refreshStore() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.context) {
                yield this.context.store.emitUpdate(this.context.namespace, this.cid);
            }
        });
    }
    set(data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.setData(this.cid, Shard.encode(data));
            yield this.refreshStore();
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    links() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const links = yield this.ipfs.object.links(this.cid);
            return Object.fromEntries(links.map(({ Name, Hash }) => [Name, Hash]));
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    addLink(name, cid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.addLink(this.cid, {
                Name: name,
                Hash: cid
            });
            yield this.refreshStore();
        });
    }
    /**
     * **Links are EXPERIMENTAL**
     */
    rmLink(cid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.cid = yield this.ipfs.object.patch.rmLink(this.cid, {
                Hash: cid
            });
            yield this.refreshStore();
        });
    }
}
exports.Shard = Shard;
class Store {
    constructor(ipfs, namespace, pubsub) {
        this.ipfs = ipfs;
        this.namespace = namespace;
        this.pubsub = pubsub;
        this.map = {};
    }
    start() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.pubsub.subscribe('$store', (msg) => {
                const { key, value } = msg.data;
                this.map[key] = cid_1.CID.parse(value);
            });
            yield this.pubsub.subscribe('$store.lookup', (msg) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                const { key } = msg.data;
                const value = this.map[key];
                if (value) {
                    yield this.pubsub.publish('$store', { key, value: value.toString() });
                    yield this.pubsub.publish(`$store/${key}`, { key, value: value.toString() });
                }
            }));
        });
    }
    emitUpdate(key, value) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.map[key] = value;
            yield this.pubsub.publish('$store', { key, value: value.toString() });
        });
    }
    /**
     * Set data to store
     *
     * @param key key
     * @param data value
     * @returns Shard
     */
    set(key, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const shard = yield Shard.create(this.ipfs, data, { namespace: key, store: this });
            this.map[key] = shard.cid;
            yield this.emitUpdate(key, shard.cid);
            return shard;
        });
    }
    /**
     * Get data from Store by key
     *
     * @param key key
     * @param timeout in ms
     * @returns Shard
     */
    get(key, timeout = 2000) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const value = this.map[key];
            if (value)
                return Shard.from(this.ipfs, value, { namespace: key, store: this });
            const peers = yield this.pubsub.peers('$store.lookup');
            if (!peers) {
                console.warn('While trying to get', key, 'no peers was found to ask for a value');
                return;
            }
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                yield this.pubsub.subscribe(`$store/${key}`, (msg) => {
                    const data = msg.data;
                    resolve(Shard.from(this.ipfs, cid_1.CID.parse(data.value), { namespace: key, store: this }));
                });
                yield this.pubsub.subscribe('$store', (msg) => {
                    const data = msg.data;
                    if (data.key === key) {
                        resolve(Shard.from(this.ipfs, cid_1.CID.parse(data.value), { namespace: key, store: this }));
                    }
                });
                yield this.pubsub.publish('$store.lookup', { key });
                setTimeout(() => {
                    console.warn('While trying to get', key, 'timeout happened');
                    resolve();
                }, timeout);
            }));
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map